"use strict";
/**
 *    Copyright 2018 Panjie Setiawan Wicaksono
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ids_1 = require("./ids");
var attachAuth = function (req, scheme, token) {
    var _a;
    var attach = scheme.attach, _name = scheme.name;
    var name = _name || 'Authorization';
    var headers = req.headers;
    var query = req.query;
    if (attach === 'header') {
        headers.set(name, token);
    }
    else {
        query =
            typeof query === 'object'
                ? tslib_1.__assign({}, query, (_a = {}, _a[name] = token, _a)) : query == null || query === ''
                ? name + "=" + token
                : query + "&" + name + "=" + token;
    }
    return tslib_1.__assign({}, req, { headers: headers, query: query });
};
var getToken = function (scheme) {
    var _prefix = scheme.prefix, tokenFn = scheme.token;
    var prefix = _prefix || '';
    if (tokenFn) {
        var token = tokenFn();
        if (token != null) {
            return "" + prefix + token;
        }
    }
    return null;
};
function requestInterceptor(scheme) {
    var authRequestInterceptor = function (input) {
        return new Promise(function (resolve) {
            var token = getToken(scheme);
            var request = input.request;
            var credentials = request.credentials;
            if (token == null) {
                // Either the tokenFn return null or it's using 'cookie' scheme
                if (process.env.NODE_ENV !== 'production') {
                    if (scheme.name !== 'cookie') {
                        // tslint:disable-next-line:no-console
                        console.warn('frest-auth: no token provided, not attaching auth');
                    }
                    else if (!credentials || credentials === 'omit') {
                        // tslint:disable-next-line:no-console
                        console.warn('frest-auth: no credentials is included, not sending cookies');
                    }
                }
                resolve(request);
                return;
            }
            var skip = request.skip;
            if (!skip || skip.indexOf(ids_1.ID_REQUEST) < 0) {
                request = attachAuth(request, scheme, token);
            }
            resolve(request);
        });
    };
    Object.defineProperty(authRequestInterceptor, 'id', { value: ids_1.ID_REQUEST });
    return authRequestInterceptor;
}
exports.default = requestInterceptor;
